"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const taskHandler_1 = require("./taskHandler");
const socket_1 = require("./socket");
/**
 * rpc framework slave part
 * @author: lizhekang
 */
class Slave extends taskHandler_1.default {
    /**
     * Master constructor
     * @param object obj with functions
     * @param config Master config
     */
    constructor(object, config) {
        super(object, config);
        this._name = config.name;
    }
    init() {
        let cfg = this._config;
        this._client = new socket_1.Client(this._name, cfg.server);
        this._client.start();
        this._client.on('rpc', this._rpc.bind(this));
    }
    call(name, params, cb, errCb) {
        cb({});
    }
    _rpc(msg) {
        let data = msg.data;
        let msgResp = new socket_1.Message('rpc', 0);
        let rpcData = JSON.parse(JSON.stringify({
            callback: data.callback,
            result: ""
        }));
        let client = this._client;
        if (typeof this._object[data.name] == 'function' && client) {
            let res = this._object[data.name].apply(this, data.params);
            if (res && res.toString() == '[object Promise]') {
                res.then((data) => {
                    rpcData.result = data;
                    msgResp.data = rpcData;
                    client.write(msgResp);
                }).catch((err) => {
                    rpcData.result = err;
                    msgResp.data = rpcData;
                    client.write(msgResp);
                });
            }
            else {
                rpcData.result = res;
                msgResp.data = rpcData;
                client.write(msgResp);
            }
        }
    }
    destroy() {
        this._client && this._client.destroy();
        this._client = null;
    }
}
exports.default = Slave;
